import locale
import tarfile
from pathlib import Path
from shutil import rmtree
from zipfile import ZipFile

import requests
from tqdm import tqdm
from unified_io import write_jsonl


def check_turn(turn: dict) -> bool:
    """Check whether a conversation turn is formatted correctly.

    Args:
        turn (dict): Conversation turn.

    Returns:
        bool: True if turn is formatted correctly.
    """
    if not "role" in turn:
        return False
    if not "content" in turn:
        return False
    if not turn["role"] in {"user", "assistant"}:
        return False
    if not isinstance(turn["content"], str):
        return False
    return True


def sanity_check(x: dict) -> dict:
    """Check whether a sample is formatted correctly.

    Args:
        x (dict): Sample.

    Returns:
        dict: Input sample.
    """
    assert "id" in x, x
    assert "label" in x, x
    assert "conversation" in x, x
    assert isinstance(x["id"], str), x
    assert isinstance(x["label"], bool), x
    assert isinstance(x["conversation"], list), x

    for turn in x["conversation"]:
        assert check_turn(turn=turn), x

    return x


def write_dataset(dataset: dict[str, list[dict]], path: Path) -> None:
    if "train" in dataset:
        write_jsonl(dataset["train"], path / "train.jsonl", callback=sanity_check)
    if "val" in dataset:
        write_jsonl(dataset["val"], path / "val.jsonl", callback=sanity_check)
    write_jsonl(dataset["test"], path / "test.jsonl", callback=sanity_check)


def remove_raw_data(path: Path):
    rmtree(path)


def download_file(url: str, filepath: str, show_progress: bool = False):
    response = requests.get(url, allow_redirects=True, stream=True)

    total_size = int(response.headers.get("content-length", 0))
    block_size = 1024

    pbar = tqdm(
        desc="Download",
        total=total_size,
        unit="B",
        unit_scale=True,
        dynamic_ncols=True,
        disable=not show_progress,
    )

    with open(filepath, "wb") as file:
        for data in response.iter_content(block_size):
            pbar.update(len(data))
            file.write(data)


def extract_zip(path: Path, filename: str):
    with ZipFile(path / filename) as zip_file:
        zip_file.extractall(path)


def extract_gzip(path: Path, filename: str):
    with tarfile.open(path / filename) as tar_file:
        tar_file.extractall(path)
